# Keyword:  `std::scoped_lock` in C++

`std::scoped_lock` is a synchronization primitive introduced in C++17 that simplifies the process of acquiring multiple locks simultaneously, helping to avoid deadlocks and ensuring exception safety. It provides a convenient way to lock one or more mutexes at the same time, and it automatically releases those locks when the std::scoped_lock object goes out of scope.

## Key Features

- Deadlock Avoidance: `std::scoped_lock` is designed to acquire multiple locks in a deadlock-free manner, which is crucial when locking multiple mutexes simultaneously.
- RAII Principle: `std::scoped_lock` follows the RAII (Resource Acquisition Is Initialization) principle, ensuring that mutexes are automatically released when the `std::scoped_lock` object is destroyed.
- Multiple Mutexes: It can lock multiple mutexes at once, making it easier to manage complex locking scenarios.

## Example: Using std::scoped_lock to Safely Lock Multiple Mutexes

Here’s an example where `std::scoped_lock` is used to safely lock two mutexes simultaneously:
```C++
#include <iostream>
#include <thread>
#include <mutex>

std::mutex mutex1;
std::mutex mutex2;

void threadFunction1() {
    for (int i = 0; i < 5; ++i) {
        std::scoped_lock lock(mutex1, mutex2);  // Lock both mutexes safely
        std::cout << "Thread 1 is executing with locks on both mutex1 and mutex2" << std::endl;
        // Critical section protected by both mutexes
    }
}

void threadFunction2() {
    for (int i = 0; i < 5; ++i) {
        std::scoped_lock lock(mutex1, mutex2);  // Lock both mutexes safely
        std::cout << "Thread 2 is executing with locks on both mutex1 and mutex2" << std::endl;
        // Critical section protected by both mutexes
    }
}

int main() {
    std::thread t1(threadFunction1);
    std::thread t2(threadFunction2);

    t1.join();
    t2.join();

    return 0;
}
```

### Explanation

- `std::scoped_lock lock(mutex1, mutex2);`: In both `threadFunction1` and `threadFunction2`, we use `std::scoped_lock` to acquire locks on `mutex1` and `mutex2` simultaneously. This ensures that both mutexes are locked in a deadlock-free manner.
- RAII: The locks are automatically released when the `std::scoped_lock` object lock goes out of scope (at the end of the loop iteration in this case), ensuring exception safety and proper resource management.

### Expected Output

The output will interleave messages from both threads, showing that they are both acquiring and releasing locks on the two mutexes safely:
```C++
Thread 1 is executing with locks on both mutex1 and mutex2
Thread 2 is executing with locks on both mutex1 and mutex2
Thread 1 is executing with locks on both mutex1 and mutex2
Thread 2 is executing with locks on both mutex1 and mutex2
```
## Why Use `std::scoped_lock`?

- Deadlock Prevention: When locking multiple mutexes, `std::scoped_lock` helps prevent deadlocks by locking the mutexes in a consistent order.
- Code Simplification: It simplifies the code by combining the locking of multiple mutexes into a single line, reducing the chances of errors.
- Automatic Unlocking: By following the RAII principle, it ensures that all locked mutexes are automatically released when the std::scoped_lock goes out of scope, reducing the risk of forgetting to unlock a mutex.

## When to Use std::scoped_lock

- Multiple Mutexes: When you need to lock multiple mutexes at the same time, especially in complex multithreading scenarios.
- Critical Sections: Protecting critical sections where multiple shared resources are involved.
- Simplifying Lock Management: When you want to simplify the management of multiple locks in your code and avoid common pitfalls like deadlocks.

## Conclusion

`std::scoped_lock` is a valuable tool in C++17 that simplifies and secures the process of locking multiple mutexes. It helps prevent deadlocks and ensures that resources are managed properly by automatically unlocking mutexes when the scope ends. This makes it an essential component in modern C++ multithreading, especially in scenarios where multiple resources need to be accessed concurrently.
